"use strict";
var TargetMetadataCollection_1 = require("../metadata/collection/TargetMetadataCollection");
var PropertyMetadataCollection_1 = require("../metadata/collection/PropertyMetadataCollection");
/**
 * Storage all metadatas of all available types: tables, fields, subscribers, relations, etc.
 * Each metadata represents some specifications of what it represents.
 */
var MetadataStorage = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function MetadataStorage() {
        // todo: type in function validation, inverse side function validation
        // todo: check on build for duplicate names, since naming checking was removed from MetadataStorage
        // todo: duplicate name checking for: table, relation, column, index, naming strategy, join tables/columns?
        // todo: check for duplicate targets too since this check has been removed too
        // -------------------------------------------------------------------------
        // Properties
        // -------------------------------------------------------------------------
        this.tableMetadatas = new TargetMetadataCollection_1.TargetMetadataCollection();
        this.namingStrategyMetadatas = new TargetMetadataCollection_1.TargetMetadataCollection();
        this.eventSubscriberMetadatas = new TargetMetadataCollection_1.TargetMetadataCollection();
        this.compositeIndexMetadatas = new TargetMetadataCollection_1.TargetMetadataCollection();
        this.columnMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
        this.relationMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
        this.joinColumnMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
        this.joinTableMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
        this.indexMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
        this.entityListenerMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
        this.relationCountMetadatas = new PropertyMetadataCollection_1.PropertyMetadataCollection();
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    /**
     * Creates a new copy of the MetadataStorage with same metadatas as in current metadata storage, but filtered
     * by classes.
     */
    MetadataStorage.prototype.mergeWithAbstract = function (allTableMetadatas, tableMetadata) {
        var _this = this;
        var compositeIndexMetadatas = this.compositeIndexMetadatas.filterByClass(tableMetadata.target);
        var columnMetadatas = this.columnMetadatas.filterByClass(tableMetadata.target);
        var relationMetadatas = this.relationMetadatas.filterByClass(tableMetadata.target);
        var joinColumnMetadatas = this.joinColumnMetadatas.filterByClass(tableMetadata.target);
        var joinTableMetadatas = this.joinTableMetadatas.filterByClass(tableMetadata.target);
        var indexMetadatas = this.indexMetadatas.filterByClass(tableMetadata.target);
        var entityListenerMetadatas = this.entityListenerMetadatas.filterByClass(tableMetadata.target);
        var relationCountMetadatas = this.relationCountMetadatas.filterByClass(tableMetadata.target);
        allTableMetadatas
            .filter(function (metadata) { return tableMetadata.isInherited(metadata); })
            .forEach(function (parentMetadata) {
            var metadatasFromAbstract = _this.mergeWithAbstract(allTableMetadatas, parentMetadata);
            columnMetadatas.push.apply(columnMetadatas, metadatasFromAbstract.columnMetadatas.filterRepeatedMetadatas(columnMetadatas));
            relationMetadatas.push.apply(relationMetadatas, metadatasFromAbstract.relationMetadatas.filterRepeatedMetadatas(relationMetadatas));
            joinColumnMetadatas.push.apply(joinColumnMetadatas, metadatasFromAbstract.joinColumnMetadatas.filterRepeatedMetadatas(joinColumnMetadatas));
            joinTableMetadatas.push.apply(joinTableMetadatas, metadatasFromAbstract.joinTableMetadatas.filterRepeatedMetadatas(joinTableMetadatas));
            indexMetadatas.push.apply(indexMetadatas, metadatasFromAbstract.indexMetadatas.filterRepeatedMetadatas(indexMetadatas));
            entityListenerMetadatas.push.apply(entityListenerMetadatas, metadatasFromAbstract.entityListenerMetadatas.filterRepeatedMetadatas(entityListenerMetadatas));
            relationCountMetadatas.push.apply(relationCountMetadatas, metadatasFromAbstract.relationCountMetadatas.filterRepeatedMetadatas(relationCountMetadatas));
        });
        return {
            compositeIndexMetadatas: compositeIndexMetadatas,
            columnMetadatas: columnMetadatas,
            relationMetadatas: relationMetadatas,
            joinColumnMetadatas: joinColumnMetadatas,
            joinTableMetadatas: joinTableMetadatas,
            indexMetadatas: indexMetadatas,
            entityListenerMetadatas: entityListenerMetadatas,
            relationCountMetadatas: relationCountMetadatas
        };
    };
    return MetadataStorage;
}());
exports.MetadataStorage = MetadataStorage;
//# sourceMappingURL=MetadataStorage.js.map