import { TableMetadata } from "./TableMetadata";
import { ColumnMetadata } from "./ColumnMetadata";
import { RelationMetadata } from "./RelationMetadata";
import { CompositeIndexMetadata } from "./CompositeIndexMetadata";
import { ForeignKeyMetadata } from "./ForeignKeyMetadata";
import { NamingStrategyInterface } from "../naming-strategy/NamingStrategyInterface";
/**
 * Contains all entity metadata.
 */
export declare class EntityMetadata {
    closureJunctionTable: EntityMetadata;
    /**
     * Naming strategy used to generate and normalize column name.
     */
    readonly namingStrategy: NamingStrategyInterface;
    readonly table: TableMetadata;
    readonly columns: ColumnMetadata[];
    readonly relations: RelationMetadata[];
    readonly compositeIndices: CompositeIndexMetadata[];
    readonly foreignKeys: ForeignKeyMetadata[];
    constructor(namingStrategy: NamingStrategyInterface, tableMetadata: TableMetadata, columnMetadatas: ColumnMetadata[], relationMetadatas: RelationMetadata[], compositeIndexMetadatas: CompositeIndexMetadata[]);
    readonly name: string;
    readonly target: Function;
    readonly oneToOneRelations: RelationMetadata[];
    readonly ownerOneToOneRelations: RelationMetadata[];
    readonly oneToManyRelations: RelationMetadata[];
    readonly manyToOneRelations: RelationMetadata[];
    readonly manyToManyRelations: RelationMetadata[];
    readonly ownerManyToManyRelations: RelationMetadata[];
    readonly relationsWithJoinColumns: RelationMetadata[];
    readonly primaryColumn: ColumnMetadata;
    readonly createDateColumn: ColumnMetadata;
    readonly updateDateColumn: ColumnMetadata;
    readonly versionColumn: ColumnMetadata;
    readonly treeChildrenCountColumn: ColumnMetadata;
    readonly treeLevelColumn: ColumnMetadata;
    readonly hasPrimaryKey: boolean;
    /**
     * Creates a new entity.
     */
    create(): any;
    createPropertiesMap(): any;
    getEntityId(entity: any): any;
    hasColumnWithPropertyName(propertyName: string): boolean;
    hasColumnWithDbName(name: string): boolean;
    findColumnWithPropertyName(propertyName: string): ColumnMetadata;
    findColumnWithDbName(name: string): ColumnMetadata;
    hasRelationWithPropertyName(propertyName: string): boolean;
    hasRelationWithDbName(dbName: string): boolean;
    findRelationWithPropertyName(propertyName: string): RelationMetadata;
    findRelationWithDbName(propertyName: string): RelationMetadata;
    findRelationWithOneWithPropertyName(propertyName: string): RelationMetadata;
    findRelationWithOneWithDbName(name: string): RelationMetadata;
    findRelationWithManyWithPropertyName(propertyName: string): RelationMetadata;
    findRelationWithManyWithDbName(name: string): RelationMetadata;
    hasRelationWithOneWithPropertyName(propertyName: string): boolean;
    hasRelationWithManyWithPropertyName(propertyName: string): boolean;
    hasRelationWithOneWithName(name: string): boolean;
    hasRelationWithManyWithName(name: string): boolean;
    readonly treeParentRelation: RelationMetadata;
    readonly treeChildrenRelation: RelationMetadata;
}
